@@ -29,9 +29,8 @@
 /// \file This file contains the definition for the class compound, encapsulating
 /// the information found for compounds in the CCD.
 
-#include <cif++.hpp>
-#include <cif++/atom_type.hpp>
-#include <cif++/point.hpp>
+#include "cif++/atom_type.hpp"
+#include "cif++/point.hpp"
 
 #include <map>
 #include <set>
