@@ -26,7 +26,9 @@
 
 #pragma once
 
-#include <cif++/row.hpp>
+#include "cif++/row.hpp"
+
+#include <array>
 
 namespace cif
 {
