@@ -26,10 +26,11 @@
 
 #pragma once
 
-#include <cif++/exports.hpp>
+#include "cif++/exports.hpp"
 
 #include <charconv>
 #include <cmath>
+#include <cstdint>
 #include <limits>
 #include <set>
 #include <sstream>
