@@ -26,8 +26,7 @@
 
 #include <cmath>
 
-#include <cif++.hpp>
-#include <cif++/atom_type.hpp>
+#include "cif++.hpp"
 
 namespace cif
 {
@@ -1028,6 +1027,9 @@ atom_type_traits::atom_type_traits(const std::string& symbol)
 			break;
 		}
 	}
+
+	if (symbol == "X")
+		m_info = &data::kKnownAtoms[0];
 	
 	if (m_info == nullptr)
 		throw std::invalid_argument("Not a known element: " + symbol);
@@ -1075,6 +1077,26 @@ bool atom_type_traits::is_metal(const std::string& symbol)
 	return result;
 }
 
+bool atom_type_traits::has_sf(int charge) const
+{
+	auto type = m_info->type;
+	if (type == D)
+		type = H;
+
+	bool result = false;
+
+	for (auto& sf: data::kWKSFData)
+	{
+		if (sf.symbol == type and sf.charge == charge)
+		{
+			result = true;
+			break;
+		}
+	}
+
+	return result;
+}
+
 auto atom_type_traits::wksf(int charge) const -> const SFData&
 {
 	auto type = m_info->type;
